//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2005, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#ifndef INST_LABEL_HH
# define INST_LABEL_HH

# include <string>
# include <ostream>

# include "misc/lockable.hh"
# include "misc/unique_string.hh"

namespace inst
{

  class Label:
    public misc::lockable
  {
  public:
    struct deref_ptr_less:
      public std::binary_function<const Label*, const Label*, bool>
    {
      bool operator()(const Label* l1, const Label* l2) const;
    };

  protected:
    Label(const std::string &s);
  public:
    Label(const std::string &s, int offset);
    virtual ~Label();

  public:
    const std::string &get_name() const;
    int get_offset() const;

  public:
    bool operator==(const Label &rhs) const;
    bool operator!=(const Label &rhs) const;
    bool operator<(const Label &rhs) const;

  protected:
    const misc::unique_string ustr_;
    int offset_;
  };

  std::ostream &
  operator<<(std::ostream &ostr, const Label &l);

} // namespace inst

# include "inst/label.hxx"

#endif // !INST_LABEL_HH
