//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#ifndef INST_LABEL_HXX
# define INST_LABEL_HXX

# include "inst/label.hh"

# include "misc/contract.hh"

namespace inst
{

  //
  inline bool
  Label::deref_ptr_less::operator()(const Label* l1, const Label* l2) const
  {
    return *l1 < *l2;
  }

  //
  inline
  Label::Label(const std::string &s):
    ustr_(s)
  {
  }

  inline
  Label::Label(const std::string &s, int offset):
    ustr_(s), offset_(offset)
  {
    lock();
  }

  //
  inline
  const std::string &
  Label::get_name() const
  {
    return *ustr_;
  }
  inline int
  Label::get_offset() const
  {
    precondition(locked());

    return offset_;
  }

  //
  inline bool
  Label::operator==(const Label &rhs) const
  {
    return ustr_ == rhs.ustr_;
  }
  inline bool
  Label::operator!=(const Label &rhs) const
  {
    return ! this->operator==(rhs);
  }
  inline bool
  Label::operator<(const Label &rhs) const
  {
    return ustr_ < rhs.ustr_;
  }

  //
  inline std::ostream &
  operator<<(std::ostream &ostr, const Label &l)
  {
    return ostr << l.get_name();
  }

} // namespace inst

#endif // !INST_LABEL_HXX
