//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2005, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#ifndef INST_SECTION_HH
# define INST_SECTION_HH

# include <string>
# include <ostream>
# include <set>

# include "misc/unique_string.hh"

# include "inst/label.hh"

namespace inst
{

  /// Abstract an assembly section
  class Section
  {
  protected:
    typedef std::set<const Label*, Label::deref_ptr_less> label_set_type;

  public:
    /// Construct an abstract section `\a name'
    Section(const std::string &name);
    virtual ~Section();

  protected:
    /** Return false if the label already exists in this section, true
	otherwise */
    bool register_label(Label *label);

  public:
    bool has_label(const misc::unique_string &label) const;
    int get_offset(const misc::unique_string &label) const;

  public:
    virtual void write(std::ostream &ostr) const = 0;

  protected:
    const misc::unique_string name_;

    label_set_type sorted_labels_;
  };

  std::ostream &
  operator<<(std::ostream &ostr, const Section &s);

} // namespace inst

# include "inst/section.hxx"

#endif // !INST_SECTION_HH
