//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2005, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#ifndef INST_TEXT_SECTION_HH
# define INST_TEXT_SECTION_HH

# include <list>
# include <vector>
# include <ostream>

# include "inst/section.hh"
# include "inst/fwd.hh"

namespace inst
{

  /// Implement an assembly text (code) section
  class TextSection:
    public Section
  {
  public:
    typedef std::list<inst::Inst*> instruction_list_type;

  public:
    TextSection(const std::string &name = ".text");
    virtual ~TextSection();

  public:
    instruction_list_type::const_iterator begin() const;
    instruction_list_type::const_iterator end() const;

  public:
    bool define_label(const misc::unique_string &id);

  public:
    void add_inst(Inst *inst);

  public:
    virtual void write(std::ostream& ostr) const;

  protected:
    std::vector<const TextLabel*> labels_;

    instruction_list_type insts_;
    instruction_list_type::iterator sentry_;
  };

} // namespace inst

# include "inst/text_section.hxx"

#endif // !INST_TEXT_SECTION_HH
