//
// unique_string.hh: string uniquifier
// Copyright (C) 2005 Benoit Perrot <benoit@lrde.epita.fr>
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// It is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#ifndef MISC_UNIQUE_STRING_HH
# define MISC_UNIQUE_STRING_HH

/** \file misc/unique_string.hh
    \brief String uniquifier.

    The idea of this class comes from TC's LRDE's Symbol class, but 
    the implementation is quite different. */

#include <string>
#include <set>
#include <iostream>

namespace misc
{

  class unique_string
  {
  private:
    typedef std::set<std::string> string_to_unique_type;

  public:
    unique_string(const std::string &str):
      it_(insert_in_pool(str))
    {}

    unique_string(const char *str):
      it_(insert_in_pool(std::string(str)))
    {}

    // Accessors
  public:
    const std::string &get() const
    {
      return *it_;
    }
    operator const std::string &() const
    {
      return *it_;
    }
    const std::string & operator* () const
    {
      return *it_;
    }

    static unsigned pool_size()
    {
      return pool_.size();
    }

    // Operators
  public:
    bool operator==(const unique_string &rhs) const
    {
      return it_ == rhs.it_;
    }
    bool operator!=(const unique_string &rhs) const
    {
      return it_ != rhs.it_;
    }
    bool operator<(const unique_string &rhs) const
    {
      if (it_ == rhs.it_)
	return false;

      return *it_ < *rhs.it_;
    }

  private:
    static string_to_unique_type::const_iterator insert_in_pool(const std::string &str);
    
  private:
    static string_to_unique_type pool_;

  private:
    const string_to_unique_type::const_iterator it_;
  };

  inline std::ostream &
  operator<<(std::ostream &ostr, const unique_string &u)
  {
    ostr << *u;
    return ostr;
  }

} // namespace misc

#endif // !MISC_UNIQUE_STRING_HH
