//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2005, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#include "common.hh"

#include "vm/nolimips_system_library.hh"
#include "vm/cpu.hh"
#include "vm/cp0.hh"
#include "vm/mmu.hh"

namespace vm
{

  NolimipsSystemLibrary::~NolimipsSystemLibrary()
  {
  }

  int
  NolimipsSystemLibrary::read(int fd, Mmu &mmu, int offset, int count)
  {
    std::istream *istr = 0;

    if (fd == 0)
      istr = &istr_;

    int i = -1;
    if (istr)
      {
 	istr->peek();

 	if (!istr->eof())
	  for (i = 0; i < count; ++i, ++offset)
	    {
	      int c = istr->get();
	      if (istr->eof())
		break;

	      mmu.data_store_byte(offset, c);
	    }
      }

    return i;
  }

  int
  NolimipsSystemLibrary::write(int fd, const Mmu &mmu, int offset, int count)
  {
    std::ostream *ostr = 0;

    if (fd == 1)
      ostr = &ostr_;
    else if (fd == 2)
      ostr = &estr_;

    int i = -1;
    if (ostr)
      {
	for (i = 0; i < count; ++i, ++offset)
	  ostr->put(static_cast<char>(mmu.data_load_byte(offset)));
      }

    return i;
  }

  void
  NolimipsSystemLibrary::invoke(Cpu &cpu)
  {
    switch (cpu.get_register(Cpu::v0))
      {
      case 0x02: // read(fd: $a0, buf: $a1, count: $a2): $v0
	{
	  cpu.set_register(Cpu::v0,
			   read(cpu.get_register(Cpu::a0),
				cpu.get_mmu(), cpu.get_register(Cpu::a1),
				cpu.get_register(Cpu::a2)));
	}
      break;

      case 0x03: // write(fd: $a0, buf: $a1, count: $a2): $v0
	{
	  cpu.set_register(Cpu::v0,
			   write(cpu.get_register(Cpu::a0),
				 cpu.get_mmu(), cpu.get_register(Cpu::a1),
				 cpu.get_register(Cpu::a2)));
	}
      break;

      case 0x06: // exit(status: $a0)
	cpu.get_cp0().set_fatal_exception();
	if (!exit_status)
	  exit_status = (exit_type) cpu.get_register(Cpu::a0);
	break;

      case 0x33: // malloc(size: $a0)
	cpu.set_register(Cpu::v0,
			 cpu.get_mmu().data_sbrk(cpu.get_register(Cpu::a0)));
	break;
      case 0x34: // free(buf: $a0)
	// FIXME
	break;

      default:
	assertion(!"syscall: Not implemented yet");
	break;
      }
  }

} // namespace vm
